@@ -7,6 +7,8 @@ import torchmetrics
 
 from sklearn.preprocessing import *
 
+import numpy as np
+
 import aiqc
 from aiqc import *
 # Still required even with `*` above.
@@ -50,13 +52,21 @@ def list_test_queues(format:str=None):
 			, 'sub_analysis': 'binary'
 			, 'datum': 'brain_tumor.csv'	
 		},
-				{
-			'queue_name': 'keras_multiclass'
-			, 'data_type': 'tabular'
+		{
+			'queue_name': 'keras_sequence_multiclass'
+			, 'data_type': 'sequence'
 			, 'supervision': 'supervised'
 			, 'analysis': 'classification'
-			, 'sub_analysis': 'multi label'
-			, 'datum': 'iris.tsv'
+			, 'sub_analysis': 'binary'
+			, 'datum': 'epilepsy.parquet'
+		},
+		{
+			'queue_name': 'keras_tabular_forecast'
+			, 'data_type': 'tabular'
+			, 'supervision': 'unsupervised'
+			, 'analysis': 'regression'
+			, 'sub_analysis': 'windowed'
+			, 'datum': 'dehli_climate.parquet'	
 		},
 		{
 			'queue_name': 'pytorch_multiclass'
@@ -112,7 +122,7 @@ For example when creating an `def test_method()... Algorithm.fn_build`
 Each test takes a slightly different approach to `fn_optimizer`.
 """
 
-# ------------------------ KERAS MULTICLASS ------------------------
+# ------------------------ KERAS TABULAR MULTICLASS ------------------------
 def keras_multiclass_fn_build(features_shape, label_shape, **hp):
 	model = keras.models.Sequential()
 	model.add(layers.Dense(units=features_shape[0], activation='relu', kernel_initializer='he_uniform'))
@@ -168,7 +178,7 @@ def make_test_queue_keras_multiclass(repeat_count:int=1, fold_count:int=None):
 	label_column = 'species'
 	label = dataset.make_label(columns=[label_column])
 
-	featureset = dataset.make_featureset(exclude_columns=[label_column])
+	feature = dataset.make_feature(exclude_columns=[label_column])
 
 	if (fold_count is not None):
 		size_test = 0.25
@@ -177,8 +187,9 @@ def make_test_queue_keras_multiclass(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 	)
@@ -191,18 +202,18 @@ def make_test_queue_keras_multiclass(repeat_count:int=1, fold_count:int=None):
 	else:
 		foldset_id = None
 
-	encoderset = featureset.make_encoderset()
+	encoderset = feature.make_encoderset()
 
-	labelcoder = label.make_labelcoder(
+	label.make_labelcoder(
 		sklearn_preprocess = OneHotEncoder(sparse=False)
 	)
 
-	fc0 = encoderset.make_featurecoder(
+	encoderset.make_featurecoder(
 		sklearn_preprocess = StandardScaler(copy=False)
 		, columns = ['petal_width']
 	)
 
-	fc1 = encoderset.make_featurecoder(
+	encoderset.make_featurecoder(
 		sklearn_preprocess = StandardScaler(copy=False)
 		, dtypes = ['float64']
 	)
@@ -222,15 +233,13 @@ def make_test_queue_keras_multiclass(repeat_count:int=1, fold_count:int=None):
 	queue = algorithm.make_queue(
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
-		, labelcoder_id = labelcoder.id
-		, encoderset_id = encoderset.id
 		, hyperparamset_id = hyperparamset.id
 		, repeat_count = repeat_count
 	)
 	return queue
 
 
-# ------------------------ KERAS BINARY ------------------------
+# ------------------------ KERAS TABULAR BINARY ------------------------
 def keras_binary_fn_build(features_shape, label_shape, **hp):
 	model = keras.models.Sequential()
 	model.add(layers.Dense(hp['neuron_count'], activation='relu', kernel_initializer='he_uniform'))
@@ -275,7 +284,7 @@ def make_test_queue_keras_binary(repeat_count:int=1, fold_count:int=None):
 	label_column = 'object'
 	label = dataset.make_label(columns=[label_column])
 
-	featureset = dataset.make_featureset(exclude_columns=[label_column])
+	feature = dataset.make_feature(exclude_columns=[label_column])
 
 	if (fold_count is not None):
 		size_test = 0.25
@@ -284,8 +293,9 @@ def make_test_queue_keras_binary(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 	)
@@ -298,13 +308,13 @@ def make_test_queue_keras_binary(repeat_count:int=1, fold_count:int=None):
 	else:
 		foldset_id = None
 
-	encoderset = featureset.make_encoderset()
-
-	labelcoder = label.make_labelcoder(
+	label.make_labelcoder(
 		sklearn_preprocess = LabelBinarizer(sparse_output=False)
 	)
 
-	fc0 = encoderset.make_featurecoder(
+	encoderset = feature.make_encoderset()
+
+	encoderset.make_featurecoder(
 		sklearn_preprocess = PowerTransformer(method='yeo-johnson', copy=False)
 		, dtypes = ['float64']
 	)
@@ -324,14 +334,85 @@ def make_test_queue_keras_binary(repeat_count:int=1, fold_count:int=None):
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
 		, hyperparamset_id = hyperparamset.id
-		, labelcoder_id = labelcoder.id
-		, encoderset_id  = encoderset.id
 		, repeat_count = repeat_count
 	)
 	return queue
 
 
-# ------------------------ KERAS REGRESSION ------------------------
+def make_test_queue_keras_text_binary(repeat_count:int=1, fold_count:int=None):
+	hyperparameters = {
+		"neuron_count": [25, 50]
+		, "epochs": [75, 150]
+	}
+
+	file_path = datum.get_path('spam.csv')
+
+	dataset = Dataset.Text.from_path(
+		file_path = file_path
+		, source_file_format = 'csv'
+		, name = 'text test dataset'
+		, dtype = None
+	)
+	
+	label_column = 'label'
+	label = dataset.make_label(columns=[label_column])
+
+	feature = dataset.make_feature(exclude_columns=[label_column])
+
+	if (fold_count is not None):
+		size_test = 0.25
+		size_validation = None
+	elif (fold_count is None):
+		size_test = 0.18
+		size_validation = 0.14
+
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
+		, size_test = size_test
+		, size_validation = size_validation
+	)
+
+	if (fold_count is not None):
+		foldset = splitset.make_foldset(
+			fold_count = fold_count
+		)
+		foldset_id = foldset.id
+	else:
+		foldset_id = None
+
+	label.make_labelcoder(
+		sklearn_preprocess = LabelBinarizer(sparse_output=False)
+	)
+
+	encoderset = feature.make_encoderset()
+
+	encoderset.make_featurecoder(
+		sklearn_preprocess = CountVectorizer(max_features = 200)
+		, columns=['TextData']
+	)
+
+	algorithm = Algorithm.make(
+		library = "keras"
+		, analysis_type = "classification_binary"
+		, fn_build = keras_binary_fn_build
+		, fn_train = keras_binary_fn_train
+	)
+
+	hyperparamset = algorithm.make_hyperparamset(
+		hyperparameters = hyperparameters
+	)
+
+	queue = algorithm.make_queue(
+		splitset_id = splitset.id
+		, foldset_id = foldset_id
+		, hyperparamset_id = hyperparamset.id
+		, repeat_count = repeat_count
+	)
+	return queue
+
+
+# ------------------------ KERAS TABULAR REGRESSION ------------------------
 def keras_regression_fn_build(features_shape, label_shape, **hp):
 	model = keras.models.Sequential()
 	model.add(layers.Dense(units=hp['neuron_count'], kernel_initializer='normal', activation='relu'))
@@ -369,19 +450,34 @@ def make_test_queue_keras_regression(repeat_count:int=1, fold_count:int=None):
 		, "epochs": [50, 75]
 	}
 
-	file_path = datum.get_path('houses.csv')
-
-	dataset = Dataset.Tabular.from_path(
-		file_path = file_path
-		, source_file_format = 'csv'
-		, name = 'real estate stats'
-		, dtype = None
-	)
+	df = datum.to_pandas('houses.csv')
+	# testing Labelpolater (we don't have a regression-sequence example yet).
+	df['price'][0] = np.NaN
+	df['price'][5] = np.NaN
+	df['price'][10] = np.NaN
+	# testing Featurepolater 2D.
+	df['nox'][5] = np.NaN
+	df['indus'][10] = np.NaN
+	df['age'][19] = np.NaN
+	
+	dataset = Dataset.Tabular.from_pandas(dataframe=df)
 	
 	label_column = 'price'
 	label = dataset.make_label(columns=[label_column])
+	label.make_labelpolater(
+		interpolate_kwargs = dict(
+			method = 'linear'
+			, limit_direction = 'both'
+			, limit_area = None
+			, axis = 0
+			, order = 1
+		)
+	)
 
-	featureset = dataset.make_featureset(exclude_columns=[label_column])
+	feature = dataset.make_feature(exclude_columns=[label_column])
+	interpolaterset = feature.make_interpolaterset()
+	interpolaterset.make_featurepolater(columns='nox')
+	interpolaterset.make_featurepolater(dtypes='float64')
 
 	if (fold_count is not None):
 		size_test = 0.25
@@ -390,8 +486,9 @@ def make_test_queue_keras_regression(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 		, bin_count = 3
@@ -406,19 +503,20 @@ def make_test_queue_keras_regression(repeat_count:int=1, fold_count:int=None):
 	else:
 		foldset_id = None
 
-	encoderset = featureset.make_encoderset()
-
-	labelcoder = label.make_labelcoder(
+	label.make_labelcoder(
 		sklearn_preprocess = PowerTransformer(method='box-cox', copy=False)
 	)
 
-	fc0 = encoderset.make_featurecoder(
+	encoderset = feature.make_encoderset()
+
+
+	encoderset.make_featurecoder(
 		include = False
 		, dtypes = ['int64']
 		, sklearn_preprocess = MinMaxScaler(copy=False)
 	)
 	# We expect double None (dtypes,columns) to use all columns because nothing is excluded.
-	fc1 = encoderset.make_featurecoder(
+	encoderset.make_featurecoder(
 		include = False
 		, dtypes = None
 		, columns = None
@@ -441,8 +539,6 @@ def make_test_queue_keras_regression(repeat_count:int=1, fold_count:int=None):
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
 		, hyperparamset_id = hyperparamset.id
-		, labelcoder_id = labelcoder.id
-		, encoderset_id = encoderset.id
 		, repeat_count = repeat_count
 	)
 	return queue
@@ -522,7 +618,7 @@ def make_test_queue_keras_image_binary(repeat_count:int=1, fold_count:int=None):
 	# Dataset.Image
 	image_urls = datum.get_remote_urls(manifest_name='brain_tumor.csv')
 	dataset_image = Dataset.Image.from_urls(urls = image_urls)
-	featureset = dataset_image.make_featureset()
+	feature = dataset_image.make_feature()
 	
 	if (fold_count is not None):
 		size_test = 0.25
@@ -531,8 +627,9 @@ def make_test_queue_keras_image_binary(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		,label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 	)
@@ -560,14 +657,232 @@ def make_test_queue_keras_image_binary(repeat_count:int=1, fold_count:int=None):
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
 		, hyperparamset_id = hyperparamset.id
-		, labelcoder_id = None
-		, encoderset_id  = None
 		, repeat_count = repeat_count
 	)
 	return queue
 
 
-# ------------------------ PYTORCH BINARY ------------------------
+# ------------------------ KERAS SEQUENCE BINARY ------------------------
+def keras_sequence_binary_fn_build(features_shape, label_shape, **hp):    
+	model = keras.models.Sequential()
+	model.add(keras.layers.LSTM(
+		hp['neuron_count']
+		, input_shape=(features_shape[0], features_shape[1])
+	))
+	model.add(keras.layers.Dense(units=label_shape[0], activation='sigmoid'))
+	return model
+
+def keras_sequence_binary_fn_train(model, loser, optimizer, samples_train, samples_evaluate, **hp):
+	model.compile(
+		loss=loser
+		, optimizer=optimizer
+		, metrics=['accuracy']
+	)
+	model.fit(
+		samples_train['features'], samples_train['labels']
+		, validation_data = (samples_evaluate['features'], samples_evaluate['labels'])
+		, verbose = 0
+		, batch_size = hp['batch_size']
+		, epochs = hp['epochs']
+		, callbacks = [keras.callbacks.History()]
+	)
+	return model
+
+def make_test_queue_keras_sequence_binary(repeat_count:int=1, fold_count:int=None):
+	df = datum.to_pandas('epilepsy.parquet')
+	# testing Featurepolater 3D.
+	df['sensor_1'][999] = np.NaN
+	df['sensor_1'][0] = np.NaN
+	df['sensor_150'][130] = np.NaN
+	df['sensor_152'][22] = np.NaN
+	df['sensor_170'][0] = np.NaN
+
+	label_df = df[['seizure']]
+	dataset_tab = aiqc.Dataset.Tabular.from_pandas(label_df)
+	label = dataset_tab.make_label(columns='seizure')
+
+	sensor_arr3D = df.drop(columns=['seizure']).to_numpy().reshape(1000,178,1).astype('float64')	
+	sensor_dataset = aiqc.Dataset.Sequence.from_numpy(sensor_arr3D)
+	feature = sensor_dataset.make_feature()
+	
+	interpolaterset = feature.make_interpolaterset()
+	interpolaterset.make_featurepolater(dtypes="float64")
+	
+	encoderset = feature.make_encoderset()
+	encoderset.make_featurecoder(
+		sklearn_preprocess = StandardScaler()
+		, columns = ['0']
+	)
+	
+	if (fold_count is not None):
+		size_test = 0.25
+		size_validation = None
+	elif (fold_count is None):
+		size_test = 0.22
+		size_validation = 0.12
+
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
+		, size_test = size_test
+		, size_validation = size_validation
+	)
+
+	if (fold_count is not None):
+		foldset = splitset.make_foldset(
+			fold_count = fold_count
+		)
+		foldset_id = foldset.id
+	else:
+		foldset_id = None
+	
+	algorithm = aiqc.Algorithm.make(
+		library = "keras"
+		, analysis_type = "classification_binary"
+		, fn_build = keras_sequence_binary_fn_build
+		, fn_train = keras_sequence_binary_fn_train
+	)
+	
+	hyperparameters = {
+		"neuron_count": [25]
+		, "batch_size": [8]
+		, "epochs": [5]
+	}
+	
+	hyperparamset = algorithm.make_hyperparamset(
+		hyperparameters = hyperparameters
+	)
+
+	queue = algorithm.make_queue(
+		splitset_id = splitset.id
+		, hyperparamset_id = hyperparamset.id
+		, repeat_count = repeat_count
+		, foldset_id = foldset_id
+	)
+	return queue
+
+
+# ------------------------ KERAS TABULAR FORECAST ------------------------
+def keras_tabular_forecast_fn_build(features_shape, label_shape, **hp):
+	model = keras.models.Sequential()
+	model.add(keras.layers.GRU(
+			hp['neuron_count']
+			, input_shape=(features_shape[0], features_shape[1])
+			, return_sequences=False
+			, activation='tanh'
+	))
+	# Automatically flattens.
+	model.add(keras.layers.Dense(label_shape[0]*label_shape[1]*hp['dense_multiplier'], activation='tanh'))
+	model.add(keras.layers.Dropout(0.3))
+	model.add(keras.layers.Dense(label_shape[0]*label_shape[1], activation='tanh'))
+	model.add(keras.layers.Dropout(0.3))
+	# Reshape to be 3D.
+	model.add(keras.layers.Reshape((label_shape[0], label_shape[1])))
+	
+	return model
+
+def keras_tabular_forecast_fn_train(model, loser, optimizer, samples_train, samples_evaluate, **hp):
+	model.compile(
+		loss=loser
+		, optimizer=optimizer
+		, metrics=['mean_squared_error']
+	)
+		
+	model.fit(
+		samples_train['features'], samples_train['features']
+		, validation_data = (samples_evaluate['features'], samples_evaluate['features'])
+		, verbose = 0
+		, batch_size = hp['batch_size']
+		, epochs = hp['epochs']
+		, callbacks = [keras.callbacks.History()]
+	)
+	return model
+
+def make_test_queue_keras_tabular_forecast(repeat_count:int=1, fold_count:int=None):
+	df = datum.to_pandas('delhi_climate.parquet')
+	df['temperature'][0] = np.NaN
+	df['temperature'][13] = np.NaN
+
+	dataset = Dataset.Tabular.from_pandas(dataframe=df)
+
+	feature = dataset.make_feature()
+
+	interpolaterset = feature.make_interpolaterset()
+	interpolaterset.make_featurepolater(dtypes=['float64'])
+
+	feature.make_window(size_window=28, size_shift=14)
+
+	encoderset = feature.make_encoderset()
+
+	encoderset.make_featurecoder(
+		sklearn_preprocess = RobustScaler(copy=False)
+		, columns = ['wind', 'pressure']
+	)
+
+	encoderset.make_featurecoder(
+		sklearn_preprocess = StandardScaler()
+		, dtypes = ['float64', 'int64']
+	)
+
+	if (fold_count is not None):
+		size_test = 0.25
+		size_validation = None
+	elif (fold_count is None):
+		size_test = 0.17
+		size_validation = 0.16
+
+	splitset = aiqc.Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = None
+		, size_test = 0.17
+		, size_validation = 0.16
+	)
+
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = None
+		, size_test = size_test
+		, size_validation = size_validation
+		, bin_count = None
+		, unsupervised_stratify_col = 'day_of_year'
+	)
+
+	if (fold_count is not None):
+		foldset = splitset.make_foldset(
+			fold_count = fold_count
+		)
+		foldset_id = foldset.id
+	else:
+		foldset_id = None
+
+	algorithm = aiqc.Algorithm.make(
+		library = "keras"
+		, analysis_type = "regression"
+		, fn_build = keras_tabular_forecast_fn_build
+		, fn_train = keras_tabular_forecast_fn_train
+	)
+
+	hyperparameters = {
+		"neuron_count": [8,10]
+		, "batch_size": [8]
+		, "epochs": [100]
+		, "dense_multiplier": [1]
+	}
+
+	hyperparamset = algorithm.make_hyperparamset(
+		hyperparameters = hyperparameters
+	)
+
+	queue = algorithm.make_queue(
+		splitset_id = splitset.id
+		, foldset_id = foldset_id
+		, hyperparamset_id = hyperparamset.id
+		, repeat_count = repeat_count
+	)
+	return queue
+
+
+# ------------------------ PYTORCH TABULAR BINARY ------------------------
 def pytorch_binary_fn_build(features_shape, label_shape, **hp):
 	model = torch.nn.Sequential(
 		nn.Linear(features_shape[0], 12),
@@ -643,7 +958,7 @@ def make_test_queue_pytorch_binary(repeat_count:int=1, fold_count:int=None):
 	label_column = 'object'
 	label = dataset.make_label(columns=[label_column])
 
-	featureset = dataset.make_featureset(exclude_columns=[label_column])
+	feature = dataset.make_feature(exclude_columns=[label_column])
 
 	if (fold_count is not None):
 		size_test = 0.25
@@ -652,8 +967,9 @@ def make_test_queue_pytorch_binary(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 	)
@@ -666,13 +982,13 @@ def make_test_queue_pytorch_binary(repeat_count:int=1, fold_count:int=None):
 	else:
 		foldset_id = None
 
-	encoderset = featureset.make_encoderset()
+	encoderset = feature.make_encoderset()
 
-	labelcoder = label.make_labelcoder(
+	label.make_labelcoder(
 		sklearn_preprocess = LabelBinarizer(sparse_output=False)
 	)
 
-	fc0 = encoderset.make_featurecoder(
+	encoderset.make_featurecoder(
 		sklearn_preprocess = PowerTransformer(method='yeo-johnson', copy=False)
 		, dtypes = ['float64']
 	)
@@ -697,14 +1013,12 @@ def make_test_queue_pytorch_binary(repeat_count:int=1, fold_count:int=None):
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
 		, hyperparamset_id = hyperparamset.id
-		, labelcoder_id = labelcoder.id
-		, encoderset_id  = encoderset.id
 		, repeat_count = repeat_count
 	)
 	return queue
 
 
-# ------------------------ PYTORCH MULTI ------------------------
+# ------------------------ PYTORCH TABULAR MULTI ------------------------
 def pytorch_multiclass_fn_build(features_shape, num_classes, **hp):
 	model = torch.nn.Sequential(
 		nn.Linear(features_shape[0], 12),
@@ -782,7 +1096,7 @@ def make_test_queue_pytorch_multiclass(repeat_count:int=1, fold_count:int=None):
 	label_column = 'species'
 	label = dataset.make_label(columns=[label_column])
 
-	featureset = dataset.make_featureset(exclude_columns=[label_column])
+	feature = dataset.make_feature(exclude_columns=[label_column])
 
 	if (fold_count is not None):
 		size_test = 0.25
@@ -791,8 +1105,9 @@ def make_test_queue_pytorch_multiclass(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 	)
@@ -805,13 +1120,13 @@ def make_test_queue_pytorch_multiclass(repeat_count:int=1, fold_count:int=None):
 	else:
 		foldset_id = None
 
-	encoderset = featureset.make_encoderset()
-
-	labelcoder = label.make_labelcoder(
+	label.make_labelcoder(
 		sklearn_preprocess = OrdinalEncoder()
 	)
 
-	fc0 = encoderset.make_featurecoder(
+	encoderset = feature.make_encoderset()
+
+	encoderset.make_featurecoder(
 		sklearn_preprocess = StandardScaler(copy=False)
 		, dtypes = ['float64']
 	)
@@ -835,15 +1150,13 @@ def make_test_queue_pytorch_multiclass(repeat_count:int=1, fold_count:int=None):
 	queue = algorithm.make_queue(
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
-		, labelcoder_id = labelcoder.id
-		, encoderset_id = encoderset.id
 		, hyperparamset_id = hyperparamset.id
 		, repeat_count = repeat_count
 	)
 	return queue
 
 
-# ------------------------ PYTORCH REGRESSION ------------------------
+# ------------------------ PYTORCH TABULAR REGRESSION ------------------------
 def pytorch_regression_lose(**hp):
 	if (hp['loss_type'] == 'mae'):
 		loser = nn.L1Loss()#mean absolute error.
@@ -851,7 +1164,7 @@ def pytorch_regression_lose(**hp):
 		loser = nn.MSELoss()
 	return loser	
 	
-def pytorch_regression_fn_build(features_shape, labels_shape, **hp):
+def pytorch_regression_fn_build(features_shape, label_shape, **hp):
 	nc = hp['neuron_count']
 	model = torch.nn.Sequential(
 		nn.Linear(features_shape[0], nc),
@@ -864,7 +1177,7 @@ def pytorch_regression_fn_build(features_shape, labels_shape, **hp):
 		nn.ReLU(),
 		nn.Dropout(p=0.4),
 
-		nn.Linear(nc, labels_shape[0])
+		nn.Linear(nc, label_shape[0])
 	)
 	return model
 
@@ -929,7 +1242,7 @@ def make_test_queue_pytorch_regression(repeat_count:int=1, fold_count:int=None):
 	label_column = 'price'
 	label = dataset.make_label(columns=[label_column])
 
-	featureset = dataset.make_featureset(exclude_columns=[label_column])
+	feature = dataset.make_feature(exclude_columns=[label_column])
 
 	if (fold_count is not None):
 		size_test = 0.25
@@ -938,8 +1251,9 @@ def make_test_queue_pytorch_regression(repeat_count:int=1, fold_count:int=None):
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 		, bin_count = 3
@@ -954,19 +1268,19 @@ def make_test_queue_pytorch_regression(repeat_count:int=1, fold_count:int=None):
 	else:
 		foldset_id = None
 
-	encoderset = featureset.make_encoderset()
-
-	labelcoder = label.make_labelcoder(
+	label.make_labelcoder(
 		sklearn_preprocess = PowerTransformer(method='box-cox', copy=False)
 	)
 
-	fc0 = encoderset.make_featurecoder(
+	encoderset = feature.make_encoderset()
+
+	encoderset.make_featurecoder(
 		include = False
 		, dtypes = ['int64']
 		, sklearn_preprocess = MinMaxScaler(copy=False)
 	)
 	# We expect double None to use all columns because nothing is excluded.
-	fc1 = encoderset.make_featurecoder(
+	encoderset.make_featurecoder(
 		include = False
 		, dtypes = None
 		, columns = None
@@ -994,8 +1308,6 @@ def make_test_queue_pytorch_regression(repeat_count:int=1, fold_count:int=None):
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
 		, hyperparamset_id = hyperparamset.id
-		, labelcoder_id = labelcoder.id
-		, encoderset_id = encoderset.id
 		, repeat_count = repeat_count
 	)
 	return queue
@@ -1050,7 +1362,7 @@ def pytorch_image_binary_fn_train(model, loser, optimizer, samples_train, sample
 	}
 
 	## --- Training loop ---
-	epochs = 40
+	epochs = 25
 	for epoch in range(epochs):
 		# --- Batch training ---
 		for i, batch in enumerate(batched_features):
@@ -1087,7 +1399,7 @@ def make_test_queue_pytorch_image_binary(repeat_count:int=1, fold_count:int=None
 	# Dataset.Image
 	image_urls = datum.get_remote_urls(manifest_name='brain_tumor.csv')
 	dataset_image = Dataset.Image.from_urls(urls = image_urls)
-	featureset = dataset_image.make_featureset()
+	feature = dataset_image.make_feature()
 	
 	if (fold_count is not None):
 		size_test = 0.25
@@ -1096,8 +1408,9 @@ def make_test_queue_pytorch_image_binary(repeat_count:int=1, fold_count:int=None
 		size_test = 0.18
 		size_validation = 0.14
 
-	splitset = featureset.make_splitset(
-		label_id = label.id
+	splitset = Splitset.make(
+		feature_ids = [feature.id]
+		, label_id = label.id
 		, size_test = size_test
 		, size_validation = size_validation
 	)
@@ -1121,8 +1434,6 @@ def make_test_queue_pytorch_image_binary(repeat_count:int=1, fold_count:int=None
 		splitset_id = splitset.id
 		, foldset_id = foldset_id
 		, hyperparamset_id = None #network takes a while.
-		, labelcoder_id = None
-		, encoderset_id  = None
 		, repeat_count = repeat_count
 	)
 	return queue
@@ -1133,10 +1444,16 @@ def make_test_queue(name:str, repeat_count:int=1, fold_count:int=None):
 		queue = make_test_queue_keras_multiclass(repeat_count, fold_count)
 	elif (name == 'keras_binary'):
 		queue = make_test_queue_keras_binary(repeat_count, fold_count)
+	elif (name == 'keras_text_binary'):
+		queue = make_test_queue_keras_text_binary(repeat_count, fold_count)
 	elif (name == 'keras_regression'):
 		queue = make_test_queue_keras_regression(repeat_count, fold_count)
 	elif (name == 'keras_image_binary'):
 		queue = make_test_queue_keras_image_binary(repeat_count, fold_count)
+	elif (name == 'keras_sequence_binary'):
+		queue = make_test_queue_keras_sequence_binary(repeat_count, fold_count)
+	elif (name == 'keras_tabular_forecast'):
+		queue = make_test_queue_keras_tabular_forecast(repeat_count, fold_count)
 	elif (name == 'pytorch_binary'):
 		queue = make_test_queue_pytorch_binary(repeat_count, fold_count)
 	elif (name == 'pytorch_multiclass'):
@@ -1146,5 +1463,5 @@ def make_test_queue(name:str, repeat_count:int=1, fold_count:int=None):
 	elif (name == 'pytorch_image_binary'):
 		queue = make_test_queue_pytorch_image_binary(repeat_count, fold_count)
 	else:
-		raise ValueError(f"\nYikes - The 'name' you specified <{name}> was not found.\nTip - Check the names in 'datum.list_test_queues()'.\n")
+		raise ValueError(f"\nYikes - The 'name' you specified <{name}> was not found.\nTip - Check the names in 'tests.list_test_queues()'.\n")
 	return queue
